export interface Tag {
    /**
     * The slug of the tag, this usually looks something like: hand_holding
     */
    slug: string,

    /**
     * An optional type of the tag
     */
    type?: string
}

export interface Post {
    /**
     * URL to the original post link
     */
    url: string,

    /**
     * A link to the full resolution image or video
     */
    contentURL?: string,

    /**
     * The optional link for the source of the image
     */
    source?: string,

    /**
     * A list of all of the tags the post has
     */
    tags?: Array<Tag>,

    /**
     * The date of the post's creation
     */
    ts?: string,
}

export enum LogType {
    ERROR = `error`,
    INFO  = `info`,
}

export interface LogEntry {
    type:   LogType,
    msg:    string,
    data:   any,
    err:    null | Error,
    ts:     Date,
}